
#include <stdio.h>
#include <stdint.h>

#include <wiringPi.h>

#include <stdlib.h>
#include <string.h>
#include <iostream>

using namespace std;


#include "myserial.h"

typedef unsigned char byte;
typedef unsigned char boolean;
#define LOW 0
#define HIGH 1
#define INPUT_PULLUP INPUT

MySerial Serial;

string XString(long int x)
{
 char buffer[32];
 sprintf(buffer,"%ld",x);
 return(string(buffer));
}


void interrupts(void)
{

}

void noInterrupts(void)
{

}

#include "SerialCommand.h"


#include "AccelStepper.h"


#ifdef __AVR__
#include <avr/signature.h>
#endif 

// Pin 13 has an LED connected on most Arduino boards.
// give it a name:
int led = 13;


typedef void (*intfn)(void);


typedef struct motorstr
{

int pin_enable;
int pin_step;
int pin_direction;
int pin_ms0;
int pin_ms1;
int pin_ms2;
int pin_input2;
int pin_input3;
int pin_relay4;
int pin_relay5;
int pin_relay6;


char * pattern;

int timode;
int basesize;

int currentstepsize;
int pendingstepsize;
int pendingstepsizechange;

long stepspeed;
long stepacceleration;

long runspeed;

int movestarted;

long movetime;
long movedistance;

long currentbaseposition;

int powermode;

AccelStepper * stepper;


int interruptmisses;
volatile unsigned long interrupt2time;
volatile unsigned long interrupt3time;
volatile int interruptflag;
volatile unsigned int  interruptdelay;

intfn interrupt2;
intfn interrupt3;

int interrupt2mode;
int interrupt3mode;


} motorstr;


#define NMOTORS 3

motorstr motor[NMOTORS];


int loopcount=0;
int loopstep=1;
int loopphase=0;

char pattern1[8]={1,1,1,1,0,0,0,0};
char pattern2[8]={1,0,0,1,0,0,0,0};







SerialCommand sCmd;



void interruptm0i0(void);
void interruptm1i0(void);
void interruptm2i0(void);


void interruptm0i1(void);
void interruptm1i1(void);
void interruptm2i1(void);




void setupvariables(void)
{
 motorstr * m;
 int i;

 for(i=0;i<NMOTORS;i++)
 {
  m=&motor[i];

  if(i==0) // Tilt
  {
   m->pin_enable = 24;
   m->pin_step=27;
   m->pin_direction=23;
   m->pin_ms0=28;
   m->pin_ms1=25;
   m->pin_ms2=29;
   m->pin_input2=8;
   m->pin_input3=8;
   m->pin_relay4=9;
   m->pin_relay5=9;
   m->pin_relay6=9;
   m->interrupt2=interruptm0i0;
   m->interrupt3=interruptm0i1;
  }
  else
  if(i==1) //Rail
  {
   m->pin_enable = 11;
   m->pin_step=14;
   m->pin_direction=10;
   m->pin_ms0=21;
   m->pin_ms1=26;
   m->pin_ms2=22;
   m->pin_input2=16;
   m->pin_input3=16;
   m->pin_relay4=1;
   m->pin_relay5=1;
   m->pin_relay6=1; 
   m->interrupt2=interruptm1i0;
   m->interrupt3=interruptm1i1;
  }
  else // Pan
  {
   m->pin_enable = 5;
   m->pin_step=3;
   m->pin_direction=4;
   m->pin_ms0=12;
   m->pin_ms1=13;
   m->pin_ms2=6;
   m->pin_input2=0;
   m->pin_input3=0;
   m->pin_relay4=2;
   m->pin_relay5=2;
   m->pin_relay6=2;
   m->interrupt2=interruptm2i0;
   m->interrupt3=interruptm2i1;
  }


  m->pattern=pattern1;

  m->timode=0;
  m->basesize=16;

  m->currentstepsize=1;
  m->pendingstepsize=1;
  m->pendingstepsizechange=0;
  m->stepspeed=16000;
  m->stepacceleration=8000;

  m->runspeed=0;

  m->movestarted=0;

  m->movetime=0;
  m->movedistance=0;

  m->currentbaseposition=0;

  m->powermode=2;

  m->stepper=new AccelStepper(1, m->pin_step, m->pin_direction);

  m->interruptmisses=0;
  m->interrupt2time=0;
  m->interrupt3time=0;
  m->interruptflag=0;
  m->interruptdelay=3;
  m->interrupt2mode=0;
  m->interrupt3mode=0;

 }
}




// ss,<number of microsteps 1,2,4,8,16>


void setupstepsize(motorstr * m,int msteps)
{
  
  if(msteps==1)
  {
   digitalWrite(m->pin_ms0,LOW);
   digitalWrite(m->pin_ms1,LOW);
   digitalWrite(m->pin_ms2,LOW);
  }
  else
  if(msteps==2)
  {
   digitalWrite(m->pin_ms0,HIGH);
   digitalWrite(m->pin_ms1,LOW);
   digitalWrite(m->pin_ms2,LOW);
  }
  else
  if(msteps==4)
  {
   digitalWrite(m->pin_ms0,LOW);
   digitalWrite(m->pin_ms1,HIGH);
   digitalWrite(m->pin_ms2,LOW);
  }
  else  
  if(msteps==8)
  {
   digitalWrite(m->pin_ms0,HIGH);
   digitalWrite(m->pin_ms1,HIGH);
   digitalWrite(m->pin_ms2,LOW);
  }
  else
  if(msteps==16)
  {
   if(m->timode)
   {
    digitalWrite(m->pin_ms0,LOW);
    digitalWrite(m->pin_ms1,LOW);
   }
   else
   {
    digitalWrite(m->pin_ms0,HIGH);
    digitalWrite(m->pin_ms1,HIGH);
   } 
   digitalWrite(m->pin_ms2,HIGH);
  }  
  else
  if(msteps==32)
  {
   digitalWrite(m->pin_ms0,HIGH);
   digitalWrite(m->pin_ms1,HIGH);
   digitalWrite(m->pin_ms2,HIGH);
  }  
}  





void flushstepsizechange(motorstr * m,long steps)
{
 if(m->pendingstepsizechange)
 {
  if(m->currentstepsize>m->pendingstepsize)
  {
   // problem, going from small steps to bigger ones
   long currentpos=m->stepper->currentPosition();
   long newpos;
   
   if(steps>=0) newpos=((currentpos+(m->currentstepsize-1)/2)*m->pendingstepsize)/m->currentstepsize;
   else         newpos=(currentpos*m->pendingstepsize)/m->currentstepsize;

//   Serial.print("cur ");   
//   Serial.print(currentpos);   
//   Serial.print("new ");      
//   Serial.println(newpos);      
   
   m->stepper->runToNewPosition(newpos*(m->currentstepsize/m->pendingstepsize));
  } 

  m->stepper->setCurrentPosition((m->stepper->currentPosition()*m->pendingstepsize)/m->currentstepsize);  
  setupstepsize(m,m->pendingstepsize);
  m->currentstepsize=m->pendingstepsize;
  m->pendingstepsizechange=0;
//  stepmultiplier=basesize/currentstepsize;
 }
//int stepmultiplier=basesize/currentstepsize;
// stepper.setMaxSpeed(stepspeed/stepmultiplier);
// stepper.setAcceleration(stepacceleration/stepmultiplier);  
 m->stepper->setMaxSpeed((m->stepspeed*m->currentstepsize)/m->basesize);
 m->stepper->setAcceleration((m->stepacceleration*m->currentstepsize)/m->basesize);  
}  



motorstr * getmotor(void)
{
 char     * arg;
 int        mn;

 arg = sCmd.next();
 if(arg!=NULL)
 {
  mn=atoi(arg);
  if(mn>=0 && mn<NMOTORS)
  {
   return(&motor[mn]);
  }
 }
 return(NULL);
}





void poweron(motorstr * m)
{
 m->pattern=pattern2;
 digitalWrite(m->pin_enable,LOW);   
 delay(25);
}

void poweroff(motorstr * m)
{
 m->pattern=pattern1; 
 delay(25);
 digitalWrite(m->pin_enable,HIGH);   
}




void powerauto(motorstr * m)
{
 if(m->powermode==2 && m->pattern==pattern2)
 {
  poweroff(m);
 } 
} 


// pw,0 off
// pw,1 on
// p2,2 auto

void cmdpower()
{
 char * arg; 
 motorstr * m;

 m=getmotor();
 if(m)
 {
  arg = sCmd.next();
  if(arg!=NULL)
  {
   m->powermode=atoi(arg);

//  Serial.print("First argument was: ");
//  Serial.println(powermode);  

   if(m->powermode==0) poweroff(m);
   else
   if(m->powermode==1) poweron(m);
   else
   {
    m->powermode=2;  
    poweroff(m); 
   }  
  }
 }
}



// mv <steps>

void cmdmove()
{
 char     * arg; 
 long       steps;
 motorstr * m;
 
 m=getmotor();
 if(m)
 {
  arg = sCmd.next();
  if(arg!=NULL)
  {
   steps=atol(arg);
//  Serial.print("move First argument was: ");
//  Serial.println(steps); 
   poweron(m);
   flushstepsizechange(m,steps);
//  steps/=stepmultiplier;  stepmultiplier=basesize/currentstepsize;

   steps=(steps*m->currentstepsize)/m->basesize;

   m->stepper->move(steps);
   m->movestarted=1;

   m->movetime=millis();
   m->movedistance=steps;
  }
 }  
}




void cmdmoveto()
{
 char * arg; 
 long   steps;
 motorstr * m;
 
 m=getmotor();
 if(m)
 {
  arg = sCmd.next();
  if(arg!=NULL)
  {
   steps=atol(arg);
   steps-=m->currentbaseposition;
//  Serial.print("First argument was: ");
//  Serial.println(steps);  
   poweron(m);

//  flushstepsizechange(steps-stepper.currentPosition()*stepmultiplier); stepmultiplier=basesize/currentstepsize;
   flushstepsizechange(m,steps-(m->stepper->currentPosition()*m->basesize)/m->currentstepsize);
  
//  steps/=stepmultiplier; stepmultiplier=basesize/currentstepsize;
   steps=(steps*m->currentstepsize)/m->basesize;
   m->stepper->moveTo(steps);
   m->movestarted=1;
  }
 }  
}




// sp <steps per second>

void  cmdspeed()
{
 char * arg; 
 motorstr * m;
 
 m=getmotor();
 if(m)
 {
  arg = sCmd.next();
  if(arg!=NULL)
  {
   m->stepspeed=atol(arg);
//  Serial.print("First argument was: ");
//  Serial.println(sspeed);  
  }
 } 
}  




void runspeedclear(motorstr * m)
{
 m->runspeed=0; 
 poweroff(m); 
 m->stepper->setCurrentPosition(m->stepper->currentPosition());    
}  

void  cmdrunspeed()
{
 char * arg; 
 long   newspeed;
 motorstr * m;
 
 m=getmotor();
 if(m)
 {
  arg = sCmd.next();
  if(arg!=NULL)
  {
   newspeed=atol(arg);
   if(newspeed==0)
   {
    runspeedclear(m);
   }
   else
   {
    if(m->runspeed==0) poweron(m);    
    m->runspeed=newspeed; 
    flushstepsizechange(m,m->runspeed);
   //stepmultiplier=basesize/currentstepsize;
   //stepper.setMaxSpeed(abs(runspeed/stepmultiplier));   
   //stepper.setSpeed(runspeed/stepmultiplier);
   
    m->stepper->setMaxSpeed(abs((m->runspeed*m->currentstepsize)/m->basesize));   
    m->stepper->setSpeed((m->runspeed*m->currentstepsize)/m->basesize);
   }
  } 
  
//  Serial.print("rs ");
//  Serial.println(runspeed/stepmultiplier);
 } 
}  




// ac,<steps per second per second>

void cmdacceleration()
{
 char * arg; 
 motorstr * m;
 
 m=getmotor();
 if(m)
 {
  arg = sCmd.next();
  if(arg!=NULL)
  {
   m->stepacceleration=atol(arg);
//  Serial.print("First argument was: ");
//  Serial.println(sacc);  
  }
 } 
}  


void cmdstop()
{
// char * arg; 
 motorstr * m;
 
 m=getmotor();
 if(m)
 {
  sCmd.next();


  m->stepper->stop();
 }
}





void  cmdsetpos()
{
 char * arg; 
// long   pos;

 motorstr * m;
 
 m=getmotor();
 if(m)
 {
  arg = sCmd.next();
 if(arg!=NULL)
  {
//   
//  currentbaseposition=atol(arg)-stepper.currentPosition()*stepmultiplier;
//int stepmultiplier=basesize/currentstepsize;
   m->currentbaseposition=atol(arg)-(m->stepper->currentPosition()*m->basesize)/m->currentstepsize;
//  Serial.print("First argument was: ");
//  Serial.println(pos);
//  pos/=stepmultiplier;
//  stepper.setCurrentPosition(pos);
  }
 } 
}  







void cmdstepsize()
{
 char * arg; 

 motorstr * m;
 
 m=getmotor();
 if(m)
 {
  arg = sCmd.next();
  if(arg!=NULL)
  {
   m->pendingstepsize=atoi(arg);
   m->pendingstepsizechange=1;
//  Serial.print("First argument was: ");
//  Serial.println(msteps);  
  }
 } 
}  



void cmdcurrentposition()
{
 motorstr * m;
 
 m=getmotor();
 if(m)
 {

// Serial.print("ps "); 
 //int stepmultiplier=basesize/currentstepsize;
//  Serial.println(stepper.currentPosition()*stepmultiplier+currentbaseposition); 
// Serial.println((stepper.currentPosition()*basesize)/currentstepsize+currentbaseposition); 
  Serial.println("ps "+XString((m->stepper->currentPosition()*m->basesize)/m->currentstepsize+m->currentbaseposition));
 }
}  


void cmddistancetogo()
{
 motorstr * m;
 
 m=getmotor();
 if(m)
 {
  Serial.print("ds "); 
 //int stepmultiplier=basesize/currentstepsize;
// Serial.println(stepper.distanceToGo()*stepmultiplier); 
  Serial.println((m->stepper->distanceToGo()*m->basesize)/m->currentstepsize);
 } 
}  




void cmdreadpin()
{
 char * arg;
 int    rpin; 
 
 arg = sCmd.next();
 if(arg!=NULL)
 {
  rpin=atoi(arg);
  
  Serial.print("rd "); 
  Serial.println(digitalRead(rpin)); 
 }
}  


void cmdwritepin()
{
 char * arg;
 int    wpin; 
 int    val;
 
 arg = sCmd.next();
 if(arg!=NULL)
 {
  wpin=atoi(arg);

  arg = sCmd.next();
  if(arg!=NULL)
  {
   val=atoi(arg);
   
   digitalWrite(wpin,val); 
  } 
 }
}  







// I am not going to use real interrupts, but the code is present
// wiringPi runs the gpio utility to set interrupts and if the mode is not valid
// strange errors will be given and the stepper task terminated


#define INTALLFLAGS2 0xF
#define INTALLFLAGS3 0xF0
#define INTARMED2 0x1
#define INTARMED3 0x10
#define INTPENDING2 0x2
#define INTPENDING3 0x20


static inline boolean attachInterruptToPin(int pin, void (*userFunc)(void), int mode)
{
// return(wiringPiISR(pin,mode,userFunc));
 return(true);
}


static inline boolean detachInterruptFromPin(int pin)
{
 return true;
}




void interrupt2(motorstr * m)
{
 if((m->interruptflag & INTALLFLAGS2)==INTARMED2)
 {
  m->interruptflag&=INTALLFLAGS3;
  m->interruptflag|=INTPENDING2;
  m->interrupt2time=millis();
  m->interruptmisses++;
//  Serial.println("int 2");
 } 
}


void interruptm0i0(void)
{
 interrupt2(&motor[0]);
}

void interruptm1i0(void)
{
 interrupt2(&motor[1]);
}

void interruptm2i0(void)
{
 interrupt2(&motor[2]);
}



void interrupt3(motorstr * m)
{
 if((m->interruptflag & INTALLFLAGS3)==INTARMED3)
 {
  m->interruptflag&=INTALLFLAGS2;
  m->interruptflag|=INTPENDING3;
  m->interrupt3time=millis();
  m->interruptmisses++;  
 } 
}


void interruptm0i1(void)
{
 interrupt3(&motor[0]);
}

void interruptm1i1(void)
{
 interrupt3(&motor[1]);
}

void interruptm2i1(void)
{
 interrupt3(&motor[2]);
}



void cleanupinterrupt(motorstr * m)
{
  int stopflag=0;

//  noInterrupts();

  if((m->interruptflag & INTALLFLAGS3)==INTPENDING3)
  {
   if(digitalRead(m->pin_input3)==m->interrupt3mode)
   {
    if((millis()-m->interrupt3time)>=m->interruptdelay)
    {
     detachInterruptFromPin(m->pin_input3);    
     m->interruptflag&=INTALLFLAGS2;
     stopflag=1;
    } 
   }
   else
   { // reset interrupt
    m->interruptflag&=INTALLFLAGS2;
    m->interruptflag|=INTARMED3;
   }  
  }
  else
  if((m->interruptflag & INTALLFLAGS3)==INTARMED3)
  {
   if(digitalRead(m->pin_input3)==m->interrupt3mode) interrupt3(m); // emulate interrupt
  }

  
  if((m->interruptflag & INTALLFLAGS2)==INTPENDING2)
  {
   if(digitalRead(m->pin_input2)==m->interrupt2mode)
   {
    if((millis()-m->interrupt2time)>=m->interruptdelay)
    {
     detachInterruptFromPin(m->pin_input2);    
     m->interruptflag&=INTALLFLAGS3;
     stopflag=1;
    } 
   }
   else
   { // reset interrupt
    m->interruptflag&=INTALLFLAGS3;
    m->interruptflag|=INTARMED2;
   }  
  }
  else
  if((m->interruptflag & INTALLFLAGS2)==INTARMED2)
  {
   if(digitalRead(m->pin_input2)==m->interrupt2mode) interrupt2(m); // emulate interrupt
  }

 // interrupts();
  
  if(stopflag)
  {
   if(m->runspeed)
   {
    runspeedclear(m);
   }
   else
  {
   m->stepper->stop();    
  } 
 }
}  





void cmdarminterrupt() // interrupt motor 0,1 mode
{
 char * arg;
 int    interruptn; 
 int    val;
 motorstr * m;
 
 m=getmotor();
 if(m)
 {
  arg = sCmd.next();
  if(arg!=NULL)
  {
   interruptn=atoi(arg);

   arg = sCmd.next();
   if(arg!=NULL)
   {
    val=atoi(arg);
  
    if(interruptn==0)
    {
     attachInterruptToPin(m->pin_input2,m->interrupt2,val);
     m->interruptflag&=INTALLFLAGS3;
     m->interruptflag|=INTARMED2;
     m->interrupt2mode=val;
    }
   
    if(interruptn==1)
    {
     attachInterruptToPin(m->pin_input3,m->interrupt3,val);
     m->interruptflag&=INTALLFLAGS2;
     m->interruptflag|=INTARMED3;
     m->interrupt3mode=val;
    }
    m->interruptmisses=0;
   }
  }
 } 
}  


 

void cmdreminterrupt()
{
 char * arg;
 int    interruptn; 
 motorstr * m;
 
 m=getmotor();
 if(m)
 {
  arg = sCmd.next();
  if(arg!=NULL)
  {
   interruptn=atoi(arg);

   if((m->interruptflag & INTALLFLAGS2))
   {
    detachInterruptFromPin(m->pin_input2);
    m->interruptflag&=INTALLFLAGS3;    
   }
   
   if((m->interruptflag & INTALLFLAGS3))
   {
    detachInterruptFromPin(m->pin_input3);
    m->interruptflag&=INTALLFLAGS2;    
   }
  } 
 } 
}  



void cmddebinterrupt()
{
 char * arg;
 motorstr * m;
 
 m=getmotor();
 if(m)
 {
  arg = sCmd.next();
  if(arg!=NULL)
  {
   m->interruptdelay=atoi(arg);
  } 
 }
}  







void cmdinterruptstate()
{
 int        i;
 motorstr * m;

 Serial.println("is");
 for(i=0;i<NMOTORS;i++)
 {
  m=&motor[i];
  Serial.print(m->interruptflag,BIN);
  Serial.print(" misses ");
  Serial.print(m->interruptmisses);
  Serial.print(" mode ");
  Serial.print(m->interrupt2mode);
  Serial.print(" ");
  Serial.println(m->interrupt3mode);
 }
}  


 
void cmdunrecognized(const char *command)
{
 Serial.print("What? ");
 Serial.println(command);
}  




void cmdversion()
{
 Serial.println( "Version: 2.03 06/June/2016 Compiled: "  __TIMESTAMP__ ", " __VERSION__); 
} 





void cmdtimode()
{
 char * arg; 
 motorstr * m;
 
 m=getmotor();
 if(m)
 {
  arg = sCmd.next();
  if(arg!=NULL) m->timode=atoi(arg);
  else          m->timode=1;

  if(m->timode==0)
  {
   if(m->currentstepsize==32)
   {
    m->pendingstepsizechange=1;
    m->pendingstepsize=16;
   }
  }
 } 
}  


void cmdbasesize()
{
 char * arg; 
 motorstr * m;
 
 m=getmotor();
 if(m)
 {
  arg = sCmd.next();
  if(arg!=NULL)
  {
   m->basesize=atoi(arg); 
  }
 }
}  



unsigned long  delaytime;

void cmddelay()
{
 char * arg;

//  Serial.println("delay1");
 
 arg = sCmd.next();
 if(arg!=NULL)
 {
  delaytime=millis()+atol(arg);
//  Serial.println("delay");
 }
}


int runcode=0;

const char * program[]={NULL,"ss 0 16","sp 0 32","mv 0 8","dl 5000","wr 4 1","dl 200","wr 4 0","dl 5000",NULL};

void runcodepoll()
{
 if(millis()>delaytime)
 {
//  Serial.println(program[runcode]); 
  if(sCmd.exec(program[runcode])==0)
  {
   runcode++;
   if(program[runcode]==NULL) runcode=1;
//   if(runcode==5) runcode=0;
  } 
 } 
}  


void cmdruncode()
{
 char * arg;
 
 arg = sCmd.next();
 if(arg!=NULL)
 {
  delaytime=0; 
  runcode=atoi(arg);
 } 
}





void cmdtest()
{
 char * arg;
 int    pin;
 int    result=1;
 int    result1;
 int    result2;
 motorstr * m;
 
 m=getmotor();
 if(m)
 {
  arg = sCmd.next();
  if(arg!=NULL)
  {
   pin=atoi(arg);
   if(pin==m->pin_input2 || pin==m->pin_input3)
   {
    Serial.print("testing sensor on pin ");
    Serial.println(pin);
    pinMode(pin,INPUT);
    pullUpDnControl (pin,PUD_DOWN);
    result1=digitalRead(pin); // expect 0 for no sensor
    Serial.print("Test 1, expected result is 0 for no sensor got ");
    Serial.println(result1);
    pullUpDnControl (pin,PUD_UP);
    delay(5);
    result2=digitalRead(pin); // expect 1 for no sensor
    Serial.print("Test 2, expected result is 1 for no sensor got ");
    Serial.println(result2);
     pinMode(pin,INPUT); 
 
    if(result1==result2) result=0;
    Serial.println("If the above two results are equal then return 0 (sensor present) else return 1");
   }
  }
 } 

 Serial.print("ts ");
 Serial.println(result); 
}  


/*
The first byte (0x1e) indicates ATMEL manufactured the chip.
The second byte indicates the amount of flash memory on the chip (0×94 = 16K, 0×95 = 32K, etc.).
The third byte indicates the µC processor.


*/


// pr 0x1e950f


void cmdprocessor()
{
 #ifdef __AVR__
 char temp[32];
 sprintf(temp,"pr 0x%02x%02x%02x",SIGNATURE_0,SIGNATURE_1,SIGNATURE_2); 
 Serial.println(temp);
 #else
 Serial.println("pr 0xFFFFFFFF"); 
 #endif
}  



void cmdquit()
{

 exit(0);
}  




// the setdown routine runs prior to closedown

void setdown(void) 
{
 motorstr * m;
 int        i;

 Serial.println("Shutting down");


#ifdef LEDSUPPORT
 pinMode(led, INPUT);
#endif


 for(i=0;i<NMOTORS;i++)
 {
  m=&motor[i];
  poweroff(m);

  pinMode(m->pin_enable,INPUT);


  pinMode(m->pin_direction,INPUT);
  pinMode(m->pin_step,INPUT);
  
  pinMode(m->pin_ms0,INPUT);
  pinMode(m->pin_ms1,INPUT);
  pinMode(m->pin_ms2,INPUT);
  
  pinMode(m->pin_input2,INPUT);
  pinMode(m->pin_input3,INPUT);

  pinMode(m->pin_relay4,INPUT);
  pinMode(m->pin_relay5,INPUT);
  pinMode(m->pin_relay6,INPUT);
 }

}






// the setup routine runs once when you press reset:
void setup() 
{
 motorstr * m;
 int        i;

 setupvariables();

#ifdef LEDSUPPORT
 pinMode(led, OUTPUT);
#endif


 for(i=0;i<NMOTORS;i++)
 {
  m=&motor[i];

  pinMode(m->pin_enable,OUTPUT);
  poweroff(m);

  pinMode(m->pin_direction,OUTPUT);
  pinMode(m->pin_step,OUTPUT);
  
  pinMode(m->pin_ms0,OUTPUT);
  pinMode(m->pin_ms1,OUTPUT);
  pinMode(m->pin_ms2,OUTPUT);
  
  setupstepsize(m,1);
  
  pinMode(m->pin_input2,INPUT);
  pinMode(m->pin_input3,INPUT);
//  pullUpDnControl (m->pin_input3,PUD_DOWN);
//  pullUpDnControl (m->pin_input2,PUD_DOWN);

  pinMode(m->pin_relay4,OUTPUT);
  pinMode(m->pin_relay5,OUTPUT);
  pinMode(m->pin_relay6,OUTPUT);
  digitalWrite(m->pin_relay4,LOW);
  digitalWrite(m->pin_relay5,LOW);  
  digitalWrite(m->pin_relay6,LOW);

 }



  
 Serial.begin(9600);

 sCmd.addCommand("pw",cmdpower);

 sCmd.addCommand("mv",cmdmove); 
 sCmd.addCommand("mt",cmdmoveto);   
  
 sCmd.addCommand("sp",cmdspeed);
 sCmd.addCommand("ac",cmdacceleration);
 
 sCmd.addCommand("ss",cmdstepsize);
 sCmd.addCommand("xx",cmdstop);
 sCmd.addCommand("ps",cmdcurrentposition);  
 sCmd.addCommand("ds",cmddistancetogo);
 sCmd.addCommand("rd",cmdreadpin);
 sCmd.addCommand("wr",cmdwritepin);
 sCmd.addCommand("ai",cmdarminterrupt);  
 sCmd.addCommand("is",cmdinterruptstate);  
 sCmd.addCommand("sc",cmdsetpos);
  
 sCmd.addCommand("rs",cmdrunspeed);
  
 sCmd.addCommand("ve",cmdversion);  
  
 sCmd.addCommand("ri",cmdreminterrupt);    
 sCmd.addCommand("ti",cmdtimode);    
 sCmd.addCommand("di",cmddebinterrupt);

 sCmd.addCommand("dl",cmddelay);
 sCmd.addCommand("rc",cmdruncode);
 sCmd.addCommand("bs",cmdbasesize);
  
 sCmd.addCommand("ts",cmdtest);

 sCmd.addCommand("pr",cmdprocessor);

 sCmd.addCommand("qu",cmdquit);
  
 sCmd.setDefaultHandler(cmdunrecognized);  
}





//int loopcheck=0;
//int looptime=0;

// the loop routine runs over and over again forever:
void loop()
{
 motorstr * m;
 int        i;

 for(i=0;i<NMOTORS;i++)
 {
  m=&motor[i];

  if(m->interruptflag & (INTALLFLAGS2|INTALLFLAGS3))
  {
   cleanupinterrupt(m); 
  }
 }

 if(Serial.available()>0) 
 {
  //    Serial.println("serial");
  sCmd.readSerial();
  return;
 }

 for(i=0;i<NMOTORS;i++)
 {
  m=&motor[i];

  if(m->runspeed)
  {
   m->stepper->runSpeed();  
  }
  else 
  if(m->stepper->distanceToGo()!=0)
  {
   m->stepper->run();
  }
  else 
  {
   if(m->movestarted)
   {
    m->movestarted=0;
    if(!runcode)
    {
     Serial.print("mv ");
     Serial.print(i);
     Serial.print(" ");
     Serial.print(m->currentbaseposition+(m->stepper->currentPosition()*m->basesize)/m->currentstepsize);
     Serial.print(" ");
     Serial.println(m->powermode);
    } 
   }
   else
   {
    powerauto(m);

    loopcount++;
    if(loopcount>20000)
    {
     loopcount=0;
#ifdef LEDSUPPORT
     digitalWrite(led, pattern[loopphase]);   // turn the LED on (HIGH is the voltage level)
#endif
     loopphase++;
     if(loopphase>7) loopphase=0;
    }  
   
    if(runcode)
    {
     runcodepoll(); 
    }   
   }
  } 
 }
}

